@@ -10,7 +10,7 @@ use warnings;
 use Carp;
 use Device::Modem;
 
-our $VERSION = '0.8';
+our $VERSION = '0.9';
 our @ISA = ("Device::Modem");
 
 =head1 NAME
@@ -596,7 +596,7 @@ sub sms_count {
 
 =over 4
 
-This method returns capacity of message stirage being used
+This method returns capacity of message storage being used
 Returns: capacity of the storagenumber of SMS, undef is storage has not been initialized
 
 Example:
@@ -616,8 +616,9 @@ sub capacity {
 
 =over 4
 
-This method returns capacity of message stirage being used
-Returns: SMS strucure, undef if unable to read or index is not valid
+This method reads sms for given index. Since CMGR is used to read SMS, the method has a side effect that  
+after its call the SMS has a READ status. 
+Returns: SMS structure, undef if unable to read or index is not valid
 
 Example:
 
@@ -681,6 +682,123 @@ sub read_sms {
 
 }
 
+
+=head2 list_sms
+
+=over 4
+
+This method lists SMS for given status. Possible status values:
+
+=over 4
+
+=item *
+
+"REC UNREAD"   	unread
+
+=item *
+
+"REC READ"   	read
+
+=item *
+
+"STO UNSENT"   	unsent
+
+=item *
+
+"STO SENT"   	sent
+
+=item *
+
+"ALL"   	all (default)
+
+=back
+
+Returns: Array of SMS strucures (the same as for read_sms, except SMSC address), empty array if no SMS for given 
+status found or unable to execute SMGL AT command
+
+Example:
+
+ my @list= $modem->list_sms();
+ print "Found ".($#list+1). " messages:\n";
+ foreach my $sms (@list)
+ {
+	print $sms->{"status"}." ".$sms->{"from"}." ".$sms->{"date_time"}." ".$sms->{"text"}."\n"\;
+ }
+
+=back
+
+=cut
+
+sub list_sms {
+	my ($self, $status) = @_;
+	if(! defined ($status))
+	{
+		$status= "ALL";
+	}
+
+	my $command="AT+CMGL=\"$status\"".Device::Modem::CR;
+        $self->_at_send($command);
+	my ($result, @lines) = $self->parse_answer(qr/OK|ERROR/, 10000);;
+
+	my @list= ();
+
+	if ($result ne "OK") 
+	{
+		carp('Failed to list SMS');
+		return @list;
+	}
+
+	my $list_size= scalar @lines;
+	if($list_size == 0)
+	{
+		return @list;
+	}
+
+#+CMGL: 0,"REC READ","+79108922481",,"15/09/10,09:56:02+12",145,4                       
+#Test                                                                                   
+#+CMGL: 1,"REC READ","111",,"15/09/13,12:06:05+12",129,134                              
+#041D04300020043D043E043C043504400435002000370039003800360037003600390034003700340039002004430441043B04430B
+#+CMGL: 2,"REC READ","111",,"15/09/13,12:06:06+12",129,80                               
+#0020043E0442043A043B044E04470435043D0430002E00200421043F0430044104380431043E002C002004470442043E002004120E
+#+CMGL: 3,"REC READ","+34611211740",,"15/10/09,18:56:58+08",145,6                       
+#Lebara 
+	for(my $i=0; $i<$list_size; $i++)
+	{
+		my %sms= ();
+		if($lines[$i] =~ /CMGL\:\s*\d+\,\s*\"?([A-Z ]+)\"?\,\s*\"(\+?\d+)\"\,.*?\,\s*\"(.+?)\"\,/)
+		{
+		$sms{"status"}= $1;
+		$sms{"from"}= $2;
+		$sms{"date_time"}= $3;
+		$sms{"smsc"}= "";
+
+		}                             
+		else
+		{
+			carp("Unable to parse CMGL output"); 
+			$i++; 
+			next;
+		}
+	
+		$i++;
+		if($i==$list_size)
+		{
+			carp "Unexpected even number of lines in CMGL output found... skipping...";
+			next;	
+
+		}
+
+		$sms{"text"}=$lines[$i];
+		push @list, \%sms;
+	
+	}
+	
+
+	return @list;
+
+}
+
+
 =head2 new_sms_count
 
 =over 4
